/************************************************************************/
/*	Copyright 1987-1991 by Chuck Musciano and Harris Corporation 	*/
/*									*/
/*	Full ownership of this software, and all rights pertaining to 	*/
/*	the for-profit distribution of this software, are retained by 	*/
/*	Chuck Musciano and Harris Corporation.  You are permitted to 	*/
/*	use this software without fee.  This software is provided "as 	*/
/*	is" without express or implied warranty.  You may redistribute 	*/
/*	this software, provided that this copyright notice is retained,	*/
/*	and that the software is not distributed for profit.  If you 	*/
/*	wish to use this software in a profit-making venture, you must 	*/
/*	first license this code and its underlying technology from 	*/
/*	Harris Corporation. 						*/
/*									*/
/*	Bottom line: you can have this software, you can use it, you 	*/
/*	can give it away.  You just can't sell any or all parts of it 	*/
/*	without prior permission from Harris Corporation. 		*/
/************************************************************************/

/************************************************************************/
/*									*/
/*	boss		main driver for Casio BOSS interface program	*/
/*									*/
/************************************************************************/

#include	<stdio.h>
#include	<ctype.h>
#include	<sys/param.h>
#include	<sys/types.h>
#include	<xview/xview.h>
#include	<xview/panel.h>
#include	<xview/font.h>
#include	"boss_ui.h"

#include	"manifest.h"
#include	"packet.h"
#include	"object.h"
#include	"globals.h"

EXPORT	int	baud_rate = BAUD_9600;
EXPORT	int	parity = NO_PARITY;
EXPORT	int	port = PORT_TTYA;
EXPORT	char	*port_file = "/dev/ttya";
EXPORT	char	*casio_file = NULL;
EXPORT	char	*appts_file = NULL;

EXPORT	Attr_attribute	INSTANCE;

EXPORT	boss_base_objects	*boss_base;

PRIVATE	Attr_attribute	CENTER_POINT;

/************************************************************************/
EXPORT	update_object_count()

{	int	i, j;
	object	*obj;
	char	buf[5];
	Font_string_dims	dims;
	static	Xv_Font	font;
	static	Xv_opaque	count[MAX_OBJECT_KINDS] = {NULL, NULL, NULL, NULL, NULL};
	static	int	amount[MAX_OBJECT_KINDS] = {0, 0, 0, 0, 0};

	if (count[0] == NULL) {
	   count[0] = boss_base->phone;
	   count[1] = boss_base->schedule;
	   count[2] = boss_base->memo;
	   count[3] = boss_base->card;
	   count[4] = boss_base->calendar;
	   font = (Xv_Font) xv_get(boss_base->main_display, PANEL_BOLD_FONT);
	   for (i = 0; i < MAX_OBJECT_KINDS; i++) {
	      xv_get(font, FONT_STRING_DIMS, (char *) xv_get(count[i], PANEL_LABEL_STRING), &dims);
	      xv_set(count[i], XV_KEY_DATA, CENTER_POINT, ((int) xv_get(count[i], XV_X)) + dims.width / 2, NULL);
	      }
	   }

	for (i = 0; i < MAX_OBJECT_KINDS; i++) {
	   for (j = 0, obj = data[i]; obj; obj = obj->next, j++)
	      ;
	   if (j != amount[i]) {
	      sprintf(buf, " %d ", j);
	      xv_get(font, FONT_STRING_DIMS, buf, &dims);
	      xv_set(count[i], PANEL_LABEL_STRING, buf, PANEL_ITEM_X, ((int) xv_get(count[i], XV_KEY_DATA, CENTER_POINT)) - dims.width / 2, 0);
	      XFlush(xv_get(boss_base->base, XV_DISPLAY));
	      amount[i] = j;
	      }
	   }
}

/************************************************************************/
main(argc, argv)

int		argc;
char		**argv;

{	char	c, *s, buf[512], getopt();
	int	pid;

	xv_init(XV_INIT_ARGC_PTR_ARGV, &argc, argv, 0);

	while ((c = getopt(&argc, argv, "b:f:p:s:", &s)) != EOF)
	   switch (c) {
	      case 'b' : if (strcmp(s, "1200") == 0)
	      		    baud_rate = BAUD_1200;
	      		 else if (strcmp(s, "2400") == 0)
	      		    baud_rate = BAUD_2400;
	      		 else if (strcmp(s, "4800") == 0)
	      		    baud_rate = BAUD_4800;
	      		 else if (strcmp(s, "9600") == 0)
	      		    baud_rate = BAUD_9600;
	      		 else {
	      		    fprintf(stderr, "%s: invalid baud rate: %s\n", argv[0], s);
	      		    exit(1);
	      		    }
	      		 break;
	      case 'f' : casio_file = strsave(s);
	      		 break;
	      case 'p' : lower(s);
	      		 if (strcmp(s, "even") == 0)
	      		    parity = EVEN_PARITY;
	      		 else if (strcmp(s, "odd") == 0)
	      		    parity = ODD_PARITY;
	      		 else if (strcmp(s, "none") == 0)
	      		    parity = NO_PARITY;
	      		 else {
	      		    fprintf(stderr, "%s: invalid parity: %s\n", argv[0], s);
	      		    exit(1);
	      		    }
	      		 break;
	      case 's' : lower(s);
	      		 if (strcmp(s, "ttya") == 0)
	      		    port = PORT_TTYA;
	      		 else if (strcmp(s, "ttyb") == 0)
	      		    port = PORT_TTYB;
	      		 else {
	      		    fprintf(stderr, "%s: invalid serial port: %s\n", argv[0], s);
	      		    exit(1);
	      		    }
	      		 break;
	      default  : fprintf(stderr, "usage: %s [-b <rate>] [-f <file>] [-p <parity>] [-s <port>]\n", argv[0]);
	      		 exit(1);
	      		 break;
	      }
	if (casio_file == NULL) {
	   sprintf(buf, "%s/.casio", getenv("HOME"));
	   casio_file = strsave(buf);
	   }
	sprintf(buf, "%s/.appointments", getenv("HOME"));
	appts_file = strsave(buf);

	pid = create_casio_handler();

	INSTANCE = xv_unique_key();
	CENTER_POINT = xv_unique_key();

	boss_base = boss_base_objects_initialize(NULL, NULL);

	update_object_count();
	notify_set_wait3_func(boss_base->base, notify_default_wait3, pid);
	window_main_loop(boss_base->base);
	destroy_casio_handler();

	exit(0);
}
