/************************************************************************/
/*	Copyright 1987-1991 by Chuck Musciano and Harris Corporation 	*/
/*									*/
/*	Full ownership of this software, and all rights pertaining to 	*/
/*	the for-profit distribution of this software, are retained by 	*/
/*	Chuck Musciano and Harris Corporation.  You are permitted to 	*/
/*	use this software without fee.  This software is provided "as 	*/
/*	is" without express or implied warranty.  You may redistribute 	*/
/*	this software, provided that this copyright notice is retained,	*/
/*	and that the software is not distributed for profit.  If you 	*/
/*	wish to use this software in a profit-making venture, you must 	*/
/*	first license this code and its underlying technology from 	*/
/*	Harris Corporation. 						*/
/*									*/
/*	Bottom line: you can have this software, you can use it, you 	*/
/*	can give it away.  You just can't sell any or all parts of it 	*/
/*	without prior permission from Harris Corporation. 		*/
/************************************************************************/

#define		RETURN(x)		return(last_token = (x))

#define		FIRST_KEYWORD		ADDRESS
#define		LAST_KEYWORD		TELEX
#define		NUM_KEYWORDS		(LAST_KEYWORD - FIRST_KEYWORD + 1)

PRIVATE	FILE	*f = NULL;
PRIVATE	int	last_token = -1;
PRIVATE	char	buf[1024];

PRIVATE	struct	{char	*name;
		 int	value;
		} token[] = {{"address",    ADDRESS},
			     {"alarm",      ALARM},
			     {"calendar",   CALENDAR},
			     {"card",       CARD},
			     {"date",       DATE},
			     {"department", DEPARTMENT},
			     {"employer",   EMPLOYER},
			     {"extra_1",    EXTRA_1},
			     {"extra_2",    EXTRA_2},
			     {"extra_3",    EXTRA_3},
			     {"extra_4",    EXTRA_4},
			     {"extra_5",    EXTRA_5},
			     {"extra_6",    EXTRA_6},
			     {"fax",        FAX},
			     {"highlight",  HIGHLIGHT},
			     {"marked",     MARKED},
			     {"memo",       MEMO},
			     {"name",       NAME},
			     {"number",     NUMBER},
			     {"pobox",      POBOX},
			     {"position",   POSITION},
			     {"schedule",   SCHEDULE},
			     {"start",      START},
			     {"stop",       STOP},
			     {"telephone",  TELEPHONE},
			     {"telex",      TELEX}};

PRIVATE	struct	{char	first;
		 char	next;
		 int	name;
		} punc[] = {{':',  '\0', COLON},
			    {'{',  '\0', LBRACE},
			    {'}',  '\0', RBRACE},
			    {'/',  '\0', SLASH},
			    {'\0', '\0', -1}};

/************************************************************************/
EXPORT	int	lex_init(path)

char	*path;

{
	if (f)
	   fclose(f);
	if (f = fopen(path, "r")) {
	   curr_file = strsave(path);
	   line_count = 1;
	   ungetc = -1;
	   return(TRUE);
	   }
	else
	   return(FALSE);
}

/************************************************************************/
PRIVATE	char	getch()

{	register	char	c;
	static		int	first = TRUE;

	if (ungetc != -1)
	   c = ungetc, ungetc = -1;
	else {
	   c = getc(f);
	   if (c == '\n')
	      line_count++;
	   }
	return(c);
}

/************************************************************************/
PRIVATE	fix_escapes(buf)

char	*buf;

{	char	*q;
	int	i;

	for (q = buf; *buf; buf++, q++)
	   if (*buf == '\\')
	      switch (*++buf) {
	         case 'b' : *q = '\010'; /* ^h */
	            	    break;
	         case 'e' : *q = '\033'; /* esc */
	          	    break;
	         case 'f' : *q = '\014'; /* ^l */
	            	    break;
	         case 'n' : *q = '\012'; /* ^j */
	            	    break;
	         case 'r' : *q = '\015'; /* ^m */
	            	    break;
	         case 't' : *q = '\011'; /* ^i */
	            	    break;
	         case '0' : 
	         case '1' : 
	         case '2' : 
	         case '3' : 
	         case '4' : 
	         case '5' : 
	         case '6' : 
	         case '7' : *q = *buf++ - '0';
	            	    for (i = 0; i < 2 && *buf >= '0' && *buf <= '7'; i++)
	            	       *q = (*q << 3) + *buf++ - '0';
	            	    buf--;
	            	    break;
	         default  : *q = *buf;
	            	    break;
	         }
	   else if (*buf == '^' && *(buf + 1) >= '@' && *(buf + 1) <= '_')
	      *q = *++buf & 0x1f;
	   else
	      *q = *buf;
	*q = '\0';
}

/************************************************************************/
PRIVATE	int	is_keyword(s)

char	*s;

{	register	int	cmp, high, low, pos;

	for (low = 0, high = NUM_KEYWORDS - 1; low <= high; )
	   if ((cmp = strcmp(s, token[pos = (high - low) / 2 + low].name)) == 0)
	      return(token[pos].value);
	   else if (cmp < 0)
	      high = pos - 1;
	   else
	      low = pos + 1;
	return(NULL);
}

/************************************************************************/
PRIVATE	int	yylex()

{	register	char	c, c1, *p;
	register	int	i, j, val;
	char			*index();
	double			atof();

	c = getch();
	while (isspace(c))
	   c = getch();
	if (isalpha(c)) {
	   p = buf;
	   *p++ = c;
	   while (isalnum(c = getch()) || c == '_')
	      *p++ = c;
	   ungetc = c;
	   *p = '\0';
	   for (p = buf; *p; p++)
	      if (isupper(*p))
	         *p = tolower(*p);
	   if (i = is_keyword(buf))
	      RETURN(i);
	   else {
	      yyerror("unexpected keyword: %s", buf);
	      RETURN(yylex());
	      }
	   }
	else if (c == '"') {
	   for (p = buf; TRUE; p++)
	      if ((*p = getch()) == '"')
	         break;
	      else if (*p == '\\')
	         *++p = getch();
	      else if (*p == '\n' || *p == '\r') {
	         yyerror("Newline in string not allowed");
	         break;
	         }
	   *p = '\0';
	   fix_escapes(buf);
	   yylval.cpval = strsave(buf);
	   RETURN(STRING);
	   }
	else if (isdigit(c)) {
	   p = buf;
	   *p++ = c;
	   while (isdigit(c = getch()))
	      *p++ = c;
	   *p = '\0';
	   ungetc = c;
	   yylval.ival = atoi(buf);
	   RETURN(INTEGER);
	   }
	else if (c == EOF) {
	   fclose(f);
	   f = NULL;
	   RETURN(EOF);
	   }
	else {
	   for (i = 0; punc[i].first; i++)
	      if (c == punc[i].first) {
	         for (c1 = getch(), j = 1; punc[i + j].first == c; j++)
	            if (c1 == punc[i + j].next)
	               RETURN(punc[i + j].name);
	         ungetc = c1;
	         RETURN(punc[i].name);
	         }
	   yyerror("Invalid character in source file: %c (0x%02x)", c, c);
	   }
	RETURN(yylex());
}

/************************************************************************/
PRIVATE	char	*get_last_token()

{	int	i;
	static	char	msg[512];

	if (last_token == INTEGER || last_token == STRING)
	   sprintf(msg, "\"%s\"", buf);
	else if (last_token >= COLON && last_token <= SLASH) {
	   for (i = 0; punc[i].first; i++)
	      if (punc[i].name == last_token) {
	         sprintf(msg, "\"%c\"", punc[i].first);
	         if (punc[i].next)
	            sprintf(msg + 2, "%c\"", punc[i].next);
	         break;
	         }
	   if (punc[i].first == '\0')
	      sprintf(msg, "!!Geez!  Some punctuation, I don't know!!");
	   }
	else if (last_token >= FIRST_KEYWORD && last_token <= LAST_KEYWORD)
	   sprintf(msg, "\"%s\"", token[last_token - FIRST_KEYWORD].name);
	else if (last_token == EOF)
	   sprintf(msg, "End Of File");
	else
	   sprintf(msg, "!!Geez!  Some keyword, I don't know!!");
	return(msg);
}
