/************************************************************************/
/*	Copyright 1987-1991 by Chuck Musciano and Harris Corporation 	*/
/*									*/
/*	Full ownership of this software, and all rights pertaining to 	*/
/*	the for-profit distribution of this software, are retained by 	*/
/*	Chuck Musciano and Harris Corporation.  You are permitted to 	*/
/*	use this software without fee.  This software is provided "as 	*/
/*	is" without express or implied warranty.  You may redistribute 	*/
/*	this software, provided that this copyright notice is retained,	*/
/*	and that the software is not distributed for profit.  If you 	*/
/*	wish to use this software in a profit-making venture, you must 	*/
/*	first license this code and its underlying technology from 	*/
/*	Harris Corporation. 						*/
/*									*/
/*	Bottom line: you can have this software, you can use it, you 	*/
/*	can give it away.  You just can't sell any or all parts of it 	*/
/*	without prior permission from Harris Corporation. 		*/
/************************************************************************/

/************************************************************************/
/*									*/
/*	object.c	manage BOSS data objects			*/
/*									*/
/************************************************************************/

#include	"manifest.h"
#include	"packet.h"
#include	"record.h"
#include	"object.h"

PRIVATE	char	*object_name[] = {"telephone", "schedule", "memo", "card", "calendar"};

/************************************************************************/
EXPORT	object	*create_object()

{	object	*obj;

	obj = (object *) malloc(sizeof(object));
	bzero(obj, sizeof(object));
	return(obj);
}

/************************************************************************/
EXPORT	free_object(obj)

object	*obj;

{	int	i;

	for (i = 0; i < MAX_FIELDS + 1; i++)
	   if (obj->field[i])
	      free(obj->field[i]);
	free(obj);
}

/************************************************************************/
EXPORT	free_object_list(obj)

object	*obj;

{	object	*o;

	while (obj) {
	   o = obj;
	   obj = obj->next;
	   free_object(o);
	   }
}

/************************************************************************/
EXPORT	object	*text_to_object(buf)

byte	*buf;

{	object	*obj;

	obj = create_object();
	obj->kind = OBJ_MEMO;
	obj->mem_memo = (byte *) strsave(buf);
	if (strlen(buf) >= MAX_OBJECT_DATA)
	   obj->mem_memo[MAX_OBJECT_DATA] = '\0';
	for (buf = obj->mem_memo; *buf; buf++)
	   if (*buf == '\n')
	      *buf = '\r';
	return(obj);
}

/************************************************************************/
EXPORT	object	*record_to_object(rec)

record	*rec;

{	object	*obj;
	byte	*p, buf[MAX_OBJECT_DATA];
	int	i;

	if (rec->type != REC_MODE)
	   return(NULL);
	obj = create_object();
	obj->from_casio = TRUE;
	bzero(buf, sizeof(buf));
	for ( ; rec; rec = rec->next)
	   switch(rec->type) {
	      case REC_DATA     : if (address_kind(rec->address) == REC_DATA_DATA)
	      			     bcopy(rec->data, buf + address_start(rec->address), rec->length);
	      			  else if (address_kind(rec->address) == REC_DATA_ALARM) {
	      			     if (obj->kind == OBJ_SCHEDULE)
	      			        ascii_to_time(rec->data, &obj->sch_alarm_time, NULL);
	      			     else {
	      			        error("alarm data encountered for %s object", object_name[obj->kind]);
	      			        free_object(obj);
	      			        return(NULL);
	      			        }
	      			     }
	      			  else if (address_kind(rec->address) == REC_DATA_BINARY) {
	      			     if (obj->kind == OBJ_CALENDAR)
	      			        obj->cal_marked = ascii_to_long(rec->data);
	      			     else {
	      			        error("binary data encountered for %s object", object_name[obj->kind]);
	      			        free_object(obj);
	      			        return(NULL);
	      			        }
	      			     }
	      			  else if (address_kind(rec->address) == REC_DATA_TIME) {
	      			     if (obj->kind == OBJ_SCHEDULE)
	      			        ascii_to_time(rec->data, &(obj->sch_start_time), &(obj->sch_stop_time));
	      			     else {
	      			        error("time data encountered for %s object", object_name[obj->kind]);
	      			        free_object(obj);
	      			        return(NULL);
	      			        }
	      			     }
	      			  else if (address_kind(rec->address) == REC_DATA_DATE) {
	      			     if (obj->kind == OBJ_SCHEDULE)
	      			        ascii_to_date(rec->data, &(obj->sch_date));
	      			     else if (obj->kind == OBJ_CALENDAR)
	      			        ascii_to_date(rec->data, &(obj->cal_date));
	      			     else {
	      			        error("date data encountered for %s object", object_name[obj->kind]);
	      			        free_object(obj);
	      			        return(NULL);
	      			        }
	      			     }
	      			  else {
	      			     error("unknown address offset (%04x) in record_to_object", address_kind(rec->address));
	      			     free_object(obj);
	      			     return(NULL);
	      			     }
	      			  break;
	      case REC_REGISTER : if (rec->address == REC_REGISTER_MARKED)
	      			     obj->marked = TRUE;
	      			  if (rec->next != NULL)
	      			     error("extra data after register record in record_to_object");
	      			  break;
	      case REC_MODE     : if (*((word *) rec->data) == REC_MODE_CALENDAR)
	      			     obj->kind = OBJ_CALENDAR;
	      			  else if (*((word *) rec->data) == REC_MODE_TELEPHONE)
	      			     obj->kind = OBJ_TELEPHONE;
	      			  else if (*((word *) rec->data) == REC_MODE_MEMO)
	      			     obj->kind = OBJ_MEMO;
	      			  else if (*((word *) rec->data) == REC_MODE_SCHEDULE) {
	      			     obj->kind = OBJ_SCHEDULE;
	      			     obj->sch_start_time.hour = INVALID_TIME;
	      			     obj->sch_stop_time.hour = INVALID_TIME;
	      			     obj->sch_alarm_time.hour = INVALID_TIME;
	      			     }
	      			  else if (*((word *) rec->data) == REC_MODE_CARD)
	      			     obj->kind = OBJ_CARD;
	      			  else {
	      			     error("unknown record mode (%04x) in record_to_object", *((word *) rec->data));
	      			     free_object(obj);
	      			     return(NULL);
	      			     }
	      			  break;
	      default		: error("unknown record type (%02x) in record_to_object", rec->type);
	      			  free_object(obj);
	      			  return(NULL);
	      }
	if (*buf) {
	   obj->field[0] = buf;
	   for (p = buf, i = 0; *p; p++)
	      if (*p == CASIO_END_ENTRY) {
	         *p = '\0';
	         obj->field[i] = (byte *) strsave(obj->field[i]);
	         obj->field[++i] = p + 1;
	         }
	   obj->field[i] = (byte *) strsave(obj->field[i]);
	   for (i++; i < MAX_FIELDS; i++)
	      obj->field[i] = NULL;
	   }
	else
	   for (i = 0; i < MAX_FIELDS; i++)
	      obj->field[i] = NULL;
	return(obj);
}

/************************************************************************/
EXPORT	record	*object_to_record(obj)

object	*obj;

{	record	*head, *rec;
	int	i, last, offset;
	byte	*p, *q;

	head = rec = make_mode_record(0); /* use a dummy mode value */
	if (obj->kind == OBJ_TELEPHONE)
	   *((word *) rec->data) = REC_MODE_TELEPHONE;
	else if (obj->kind == OBJ_MEMO)
	   *((word *) rec->data) = REC_MODE_MEMO;
	else if (obj->kind == OBJ_CARD)
	   *((word *) rec->data) = REC_MODE_CARD;
	else if (obj->kind == OBJ_SCHEDULE) {
	   *((word *) rec->data) = REC_MODE_SCHEDULE;
	   rec->next = make_date_record(&(obj->sch_date));
	   rec = rec->next;
	   if (obj->sch_start_time.hour != INVALID_TIME) {
	      rec->next = make_time_record(REC_DATA_TIME, &(obj->sch_start_time), &(obj->sch_stop_time));
	      rec = rec->next;
	      }
	   if (obj->sch_alarm_time.hour != INVALID_TIME) {
	      rec->next = make_time_record(REC_DATA_ALARM, &(obj->sch_alarm_time), NULL);
	      rec = rec->next;
	      }
	   }
	else if (obj->kind == OBJ_CALENDAR) {
	   *((word *) rec->data) = REC_MODE_CALENDAR;
	   rec->next = make_date_record(&(obj->cal_date));
	   rec = rec->next;
	   rec->next = create_record();
	   rec = rec->next;
	   rec->type = REC_DATA;
	   rec->address = REC_DATA_BINARY;
	   rec->length = 4;
	   long_to_ascii(obj->cal_marked, rec->data);
	   }
	else {
	   error("unknown object (%d) passed to object_to_record", obj->kind);
	   free_record(head);
	   return(NULL);
	   }
	for (last = MAX_FIELDS; last >= 0 && obj->field[last] == NULL; last--)
	   ;
	for (i = offset = 0; i <= last; i++) {
	   p = obj->field[i];
	   do {
	      rec->next = create_record();
	      rec = rec->next;
	      rec->type = REC_DATA;
	      rec->address = REC_DATA_DATA + offset;
	      if (p == NULL)
	         rec->length = 0;
	      else if (q = (byte *) index(p, CASIO_END_LINE))
	         rec->length = q - p + 1;
	      else
	         rec->length = strlen(p);
	      if (rec->length > 127)
	         rec->length = 127;
	      if (p) {
	         bcopy(p, rec->data, rec->length);
	         p += rec->length;
	         }
	      if ((p == NULL || *p == '\0') && i < last)
	         rec->data[rec->length++] = CASIO_END_ENTRY;
	      offset += rec->length;
	      } while (p && *p);
	   }
	rec->next = create_record();
	rec = rec->next;
	rec->length = 0;
	rec->type = REC_REGISTER;
	rec->address = obj->marked? REC_REGISTER_MARKED : REC_REGISTER_NORMAL;
	return(head);
}

/************************************************************************/
EXPORT	int	send_object(obj)

object	*obj;

{	record	*rec, *r;
	int	result, retries;

	for (retries = 4; retries > 0; retries--) {
	   rec = object_to_record(obj);
	   if (!send_record(rec))
	      return(FALSE);
	   while (rec) {
	      r = rec;
	      rec = rec->next;
	      free_record(r);
	      }
	   if ((result = wait_for_acknowledgement()) == OBJECT_RECEIVED)
	      return(TRUE);
	   else if (result == OBJECT_RESEND)
	      wait_casio(3000); /* let's all catch our breath */
	   else {
	      error("BOSS did not acknowledge receipt of object");
	      return(FALSE);
	      }
	   }
	error("Transmission aborted due to repeated failures reported by BOSS");
	return(FALSE);
}

/************************************************************************/
EXPORT	object	*receive_object()

{	record	*rec, *r;
	object	*obj;

	if (rec = receive_record()) {
	   acknowledge_object();
	   obj = record_to_object(rec);
	   while (rec) {
	      r = rec;
	      rec = rec->next;
	      free_record(r);
	      }
	   return(obj);
	   }
	else
	   return(NULL);
}
