/************************************************************************/
/*	Copyright 1987-1991 by Chuck Musciano and Harris Corporation 	*/
/*									*/
/*	Full ownership of this software, and all rights pertaining to 	*/
/*	the for-profit distribution of this software, are retained by 	*/
/*	Chuck Musciano and Harris Corporation.  You are permitted to 	*/
/*	use this software without fee.  This software is provided "as 	*/
/*	is" without express or implied warranty.  You may redistribute 	*/
/*	this software, provided that this copyright notice is retained,	*/
/*	and that the software is not distributed for profit.  If you 	*/
/*	wish to use this software in a profit-making venture, you must 	*/
/*	first license this code and its underlying technology from 	*/
/*	Harris Corporation. 						*/
/*									*/
/*	Bottom line: you can have this software, you can use it, you 	*/
/*	can give it away.  You just can't sell any or all parts of it 	*/
/*	without prior permission from Harris Corporation. 		*/
/************************************************************************/

%{

#include	<stdio.h>
#include	<ctype.h>

#include	"manifest.h"
#include	"object.h"

#define		range(val, l, h)	((val) >= (l) && (val) <= (h))

EXPORT	object	*parsed_object_list;
EXPORT	int	parse_errors_occured;

PRIVATE	char	*get_last_token();

PRIVATE	char	*curr_file;
PRIVATE	int	line_count = 1;
PRIVATE	char	ungetc = -1;

PRIVATE	time	t_buf;
PRIVATE	date	d_buf;
PRIVATE	object	*curr_object;

%}

%start	boss_data

%union	{char		*cpval;
	 date		*dval;
	 int		ival;
	 object		*oval;
	 time		*tval;
	}

%token	<cpval>	STRING
%token	<ival>	INTEGER

%token		COLON LBRACE RBRACE SLASH

%token		ADDRESS ALARM CALENDAR CARD DATE DEPARTMENT EMPLOYER
		EXTRA_1 EXTRA_2 EXTRA_3 EXTRA_4 EXTRA_5 EXTRA_6 FAX HIGHLIGHT MARKED
		MEMO NAME NUMBER POBOX POSITION SCHEDULE START STOP TELEPHONE TELEX

%type	<cpval>	address department employer extra_1 extra_2 extra_3 extra_4 extra_5 extra_6
		fax memo name number pobox position telex
%type	<dval>	date date_val
%type	<ival>	highlight int_list
%type	<oval>	calendar_object card_object memo_object object object_list phone_object schedule_object
%type	<tval>	alarm_time start_time stop_time time

%%

boss_data	:	object_list
		;

object_list	:	empty
					{ $$ = parsed_object_list = NULL; }
		|	object_list object
					{ if ($1 == NULL)
					     parsed_object_list = $2;
					  else
					     $1->next = $2;
					  $$ = $2;
					}
		;

object		:	calendar_object
		|	card_object
		|	memo_object
		|	phone_object
		|	schedule_object
		;

phone_object	:	LBRACE TELEPHONE
					{ curr_object = create_object();
					  curr_object->kind = OBJ_TELEPHONE;
					}
			phone_list RBRACE
					{ if (curr_object->pho_name == NULL || curr_object->pho_name[0] == '\0')
					     yyerror("missing name in phone object");
					  $$ = curr_object;
					}
		;

phone_list	:	empty
		|	phone_list phone_attr
		;

phone_attr	:	marked
					{ curr_object->marked = TRUE; }
		|	name
					{ curr_object->pho_name = (byte *) $1; }
		|	number
					{ curr_object->pho_number = (byte *) $1; }
		|	address
					{ curr_object->pho_address = (byte *) $1; }
		|	extra_1
					{ curr_object->pho_extra_1 = (byte *) $1; }
		|	extra_2
					{ curr_object->pho_extra_2 = (byte *) $1; }
		|	extra_3
					{ curr_object->pho_extra_3 = (byte *) $1; }
		|	extra_4
					{ curr_object->pho_extra_4 = (byte *) $1; }
		|	extra_5
					{ curr_object->pho_extra_5 = (byte *) $1; }
		|	extra_6
					{ curr_object->pho_extra_6 = (byte *) $1; }
		;

schedule_object	:	LBRACE SCHEDULE
					{ curr_object = create_object();
					  curr_object->kind = OBJ_SCHEDULE;
					  curr_object->sch_start_time.hour = INVALID_TIME;
					  curr_object->sch_stop_time.hour = INVALID_TIME;
					  curr_object->sch_alarm_time.hour = INVALID_TIME;
					  curr_object->sch_date.month = INVALID_TIME;
					}
			schedule_list RBRACE
					{ if (curr_object->sch_date.month == INVALID_TIME)
					     yyerror("missing date in schedule object");
					  $$ = curr_object;
					}
		;

schedule_list	:	empty
		|	schedule_list schedule_attr
		;

schedule_attr	:	marked
					{ curr_object->marked = TRUE; }
		|	date
					{ curr_object->sch_date = *$1; }
		|	start_time
					{ curr_object->sch_start_time = *$1; }
		|	stop_time
					{ curr_object->sch_stop_time = *$1; }
		|	alarm_time
					{ curr_object->sch_alarm_time = *$1; }
		|	memo
					{ curr_object->sch_memo = (byte *) $1; }
		;

memo_object	:	LBRACE MEMO
					{ curr_object = create_object();
					  curr_object->kind = OBJ_MEMO;
					}
			memo_list RBRACE
					{ if (curr_object->mem_memo == NULL || curr_object->mem_memo[0] == '\0')
					     yyerror("missing memo in memo object");
					  $$ = curr_object;
					}
		;

memo_list	:	empty
		|	memo_list memo_attr
		;

memo_attr	:	marked
					{ curr_object->marked = TRUE; }
		|	memo
					{ curr_object->sch_memo = (byte *) $1; }
		;

card_object	:	LBRACE CARD
					{ curr_object = create_object();
					  curr_object->kind = OBJ_CARD;
					}
			card_list RBRACE
					{ if (curr_object->car_employer == NULL || curr_object->car_employer[0] == '\0')
					     yyerror("missing employer in card object");
					  if (curr_object->car_name == NULL || curr_object->car_name[0] == '\0')
					     yyerror("missing name in card object");
					  $$ = curr_object;
					}
		;

card_list	:	empty
		|	card_list card_attr
		;

card_attr	:	marked
					{ curr_object->marked = TRUE; }
		|	employer
					{ curr_object->car_employer = (byte *) $1; }
		|	name
					{ curr_object->car_name = (byte *) $1; }
		|	number
					{ curr_object->car_number = (byte *) $1; }
		|	position
					{ curr_object->car_position = (byte *) $1; }
		|	department
					{ curr_object->car_department = (byte *) $1; }
		|	pobox
					{ curr_object->car_pobox = (byte *) $1; }
		|	address
					{ curr_object->car_address = (byte *) $1; }
		|	telex
					{ curr_object->car_telex = (byte *) $1; }
		|	fax
					{ curr_object->car_fax = (byte *) $1; }
		|	extra_1
					{ curr_object->car_extra_1 = (byte *) $1; }
		|	extra_2
					{ curr_object->car_extra_2 = (byte *) $1; }
		|	extra_3
					{ curr_object->car_extra_3 = (byte *) $1; }
		|	extra_4
					{ curr_object->car_extra_4 = (byte *) $1; }
		|	extra_5
					{ curr_object->car_extra_5 = (byte *) $1; }
		|	extra_6
					{ curr_object->car_extra_6 = (byte *) $1; }
		;

calendar_object	:	LBRACE CALENDAR
					{ curr_object = create_object();
					  curr_object->kind = OBJ_CALENDAR;
					  curr_object->cal_date.month = INVALID_TIME;
					}
			calendar_list RBRACE
					{ if (curr_object->cal_date.month == INVALID_TIME)
					     yyerror("missing date in calendar object");
					  $$ = curr_object;
					}
		;

calendar_list	:	empty
		|	calendar_list calendar_attr
		;

calendar_attr	:	date
					{ curr_object->cal_date = *$1; }
		|	highlight
					{ curr_object->cal_marked |= $1; }
		;

address		:	LBRACE ADDRESS STRING RBRACE
					{ $$ = $3; }
		;

alarm_time	:	LBRACE ALARM time RBRACE
					{ $$ = $3; }
		;

date		:	LBRACE DATE date_val RBRACE
					{ $$ = $3; }
		;

department	:	LBRACE DEPARTMENT STRING RBRACE
					{ $$ = $3; }
		;

employer	:	LBRACE EMPLOYER STRING RBRACE
					{ $$ = $3; }
		;

extra_1		:	LBRACE EXTRA_1 STRING RBRACE
					{ $$ = $3; }
		;

extra_2		:	LBRACE EXTRA_2 STRING RBRACE
					{ $$ = $3; }
		;

extra_3		:	LBRACE EXTRA_3 STRING RBRACE
					{ $$ = $3; }
		;

extra_4		:	LBRACE EXTRA_4 STRING RBRACE
					{ $$ = $3; }
		;

extra_5		:	LBRACE EXTRA_5 STRING RBRACE
					{ $$ = $3; }
		;

extra_6		:	LBRACE EXTRA_6 STRING RBRACE
					{ $$ = $3; }
		;

fax		:	LBRACE FAX STRING RBRACE
					{ $$ = $3; }
		;

highlight	:	LBRACE HIGHLIGHT int_list RBRACE
					{ $$ = $3; }
		;

marked		:	LBRACE MARKED RBRACE
		;

memo		:	LBRACE MEMO STRING RBRACE
					{ $$ = $3; }
		;

name		:	LBRACE NAME STRING RBRACE
					{ $$ = $3; }
		;

number		:	LBRACE NUMBER STRING RBRACE
					{ $$ = $3; }
		;

pobox		:	LBRACE POBOX STRING RBRACE
					{ $$ = $3; }
		;

position	:	LBRACE POSITION STRING RBRACE
					{ $$ = $3; }
		;

start_time	:	LBRACE START time RBRACE
					{ $$ = $3; }
		;

stop_time	:	LBRACE STOP time RBRACE
					{ $$ = $3; }
		;

telex		:	LBRACE TELEX STRING RBRACE
					{ $$ = $3; }
		;

time		:	INTEGER COLON INTEGER
					{ if (range($1, 0, 23) && range($3, 0, 59)) {
					     t_buf.hour = $1;
					     t_buf.minute = $3;
					     }
					  else
					     yyerror("invalid time: %2d:%02d", $1, $3);
					  $$ = &t_buf;
					}
		;

date_val	:	INTEGER SLASH INTEGER SLASH INTEGER
					{ if (range($1, 1, 12) && range($3, 1, 31) && (range($5, 1900, 2099) || range($5, 0, 99))) {
					     d_buf.month = $1;
					     d_buf.day = $3;
					     if (range($5, 0, 99))
					        d_buf.year = 1900 + $5;
					     else
					        d_buf.year = $5;
					     }
					  else
					     yyerror("invalid date: %2d/%02d/%02d", $1, $3, $5);
					  $$ = &d_buf;
					}

int_list	: empty
					{ $$ = 0; }
		| int_list INTEGER
					{ if (range($2, 1, 31))
					     $$ = $1 | (1 << ($2 - 1));
					  else {
					     yyerror("invalid highlight date: %d", $2);
					     $$ = 0;
					     }
					}
		;

empty		: ;

%%

/************************************************************************/
PRIVATE	yyerror(s1, s2, s3, s4, s5, s6, s7)

char	*s1, *s2, *s3, *s4, *s5, *s6, *s7;

{	char	buf1[1024], buf2[1024];

	sprintf(buf1, "%s: line %d: ", curr_file, line_count - ((ungetc == '\n')? 1 : 0));
	sprintf(buf2, s1, s2, s3, s4, s5, s6, s7);
	strcat(buf1, buf2);
	if (strcmp(s1, "syntax error") == 0) {
	   strcat(buf1, " at or near ");
	   strcat(buf1, get_last_token());
	   }
	error(buf1);
	yyclearin;
	parse_errors_occured++;
}

#include "lex.c"
