/************************************************************************/
/*	Copyright 1987-1991 by Chuck Musciano and Harris Corporation 	*/
/*									*/
/*	Full ownership of this software, and all rights pertaining to 	*/
/*	the for-profit distribution of this software, are retained by 	*/
/*	Chuck Musciano and Harris Corporation.  You are permitted to 	*/
/*	use this software without fee.  This software is provided "as 	*/
/*	is" without express or implied warranty.  You may redistribute 	*/
/*	this software, provided that this copyright notice is retained,	*/
/*	and that the software is not distributed for profit.  If you 	*/
/*	wish to use this software in a profit-making venture, you must 	*/
/*	first license this code and its underlying technology from 	*/
/*	Harris Corporation. 						*/
/*									*/
/*	Bottom line: you can have this software, you can use it, you 	*/
/*	can give it away.  You just can't sell any or all parts of it 	*/
/*	without prior permission from Harris Corporation. 		*/
/************************************************************************/

/************************************************************************/
/*									*/
/*	record.c	handles converting records to/from packets	*/
/*									*/
/************************************************************************/

#include	"manifest.h"
#include	"packet.h"
#include	"record.h"

/************************************************************************/
EXPORT	record	*create_record()

{	record	*rec;

	rec = (record *) malloc(sizeof(record));
	bzero(rec, sizeof(record));
	return(rec);
}

/************************************************************************/
EXPORT	free_record(rec)

record	*rec;

{
	free(rec);
}

/************************************************************************/
EXPORT	record	*make_mode_record(mode)

word	mode;

{	record	*rec;

	rec = create_record();
	rec->type = REC_MODE;
	rec->address = 0;
	rec->length = 2;
	*((word *) rec->data) = mode;
	return(rec);
}

/************************************************************************/
EXPORT	record	*make_date_record(d)

date	*d;

{	record	*rec;

	rec = create_record();
	rec->type = REC_DATA;
	rec->address = REC_DATA_DATE;
	date_to_ascii(d, rec->data);
	rec->length = strlen(rec->data);
	return(rec);
}

/************************************************************************/
EXPORT	record	*make_time_record(addr, start, stop)

word	addr;
time	*start;
time	*stop;

{	record	*rec;

	rec = create_record();
	rec->type = REC_DATA;
	rec->address = addr;
	time_to_ascii(start, stop, rec->data);
	rec->length = strlen(rec->data);
	return(rec);
}

/************************************************************************/
EXPORT	send_record(rec)

record	*rec;

{
	for ( ; rec; rec = rec->next)
	   if (!send_packet(rec))
	      return(FALSE);
	return(TRUE);
}

/************************************************************************/
EXPORT	record	*receive_record()

{	record	*head, *rec;
	static	word	last_mode = REC_MODE_INVALID;

	head = rec = create_record();
	if (!receive_packet(head))
	   return(NULL);
	if (head->type == REC_EXIT) {
	   free_record(head);
	   return(NULL);
	   }
	else if (head->type == REC_MODE)
	   last_mode = *((word *) head->data);
	else {
	   if (last_mode != REC_MODE_INVALID) {
	      rec = make_mode_record(last_mode);
	      rec->next = head;
	      head = rec;
	      rec = head->next;
	      }
	   else
	      error("no mode specified in receive_record");
	   }
	while (TRUE) {
	   rec->next = create_record();
	   rec = rec->next;
	   if (!receive_packet(rec)) {
	      error("premature end of packet sequence in receive_record");
	      return(NULL);
	      }
	   else if (rec->type == REC_REGISTER)
	      break;
	   else if (rec->type != REC_DATA) {
	      error("invalid record encountered (%02x) in receive_record", rec->type);
	      return(NULL);
	      }
	   }
	return(head);
}
